@@ -6,7 +6,7 @@
 ;; Keywords: tools, wp
 ;; URL: https://github.com/astoff/jit-spell
 ;; Package-Requires: ((emacs "27.1") (compat "29.1"))
-;; Version: 0.2
+;; Version: 0.3
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -50,6 +50,7 @@
 (require 'cl-lib)
 (require 'compat)
 (require 'ispell)
+(require 'text-property-search)
 
 (defgroup jit-spell nil
   "Check spelling as you type."
@@ -105,14 +106,18 @@ to false positives when it is used as a quotation mark."
                  (const :tag "Yes" t)
                  (const :tag "No" nil)))
 
-(defvar jit-spell-delayed-commands nil)
-(make-obsolete-variable 'jit-spell-delayed-commands "Not necessary anymore" "0.2")
+(defcustom jit-spell-correct-next-key 'auto
+  "Key used to jump to the next misspelling in `jit-spell-correct-word'.
+This can be a key sequence, nil, or `auto' to use the same key to
+which the command `jit-spell-correct-word' is bound."
+  :type '(choice (const :tag "Determine automatically" auto)
+                 (string :tag "Key")
+                 (const :tag "None" nil)))
 
 (defvar jit-spell--ignored-p #'jit-spell--default-ignored-p
   "Predicate satisfied by words to ignore.
 It should be a function taking two arguments, the start and end
 positions of the word.")
-(make-obsolete-variable 'jit-spell-ignored-p 'jit-spell--ignored-p "0.2")
 
 (defvar jit-spell--filter-region #'jit-spell--filter-region
   "Function to extract regions of interest from the buffer.
@@ -134,45 +139,22 @@ move the point with impunity.")
 (defvar-local jit-spell--local-words nil
   "A list of words accepted temporarily in this buffer.")
 
-;;; Mode-specific support
-
-(defun jit-spell--default-ignored-p (start end)
-  "Return non-nil if word between START and END should not be spell-checked."
-  (or (get-text-property start 'jit-spell-ignored)
-      (let ((face (get-text-property start 'face)))
-        (if (listp face)
-            (seq-some (lambda (f) (memq f jit-spell-ignored-faces))
-                      face)
-          (memq face jit-spell-ignored-faces)))
-      (member (buffer-substring-no-properties start end)
-              jit-spell--local-words)))
-
-(defun jit-spell--prog-ignored-p (start _end)
-  "Additional ignore predicate for `prog-mode'."
-  (let ((face (get-text-property start 'face)))
-    (not (if (listp face)
-             (seq-some (lambda (f) (memq f jit-spell-prog-mode-faces))
-                       face)
-           (memq face jit-spell-prog-mode-faces)))))
-
 ;;; Overlays
 
 (put 'jit-spell 'evaporate t)
+(put 'jit-spell 'face 'jit-spell-misspelling)
 
 (defun jit-spell--make-overlay (start end corrections)
   "Make an overlay to highlight incorrect word between START and END.
 Also add the list of CORRECTIONS as a property."
   (let ((ov (make-overlay start end nil t)))
-    (overlay-put ov 'category 'jit-spell)
     (overlay-put ov 'jit-spell-corrections corrections)
     (if (not (<= start (point) end))
-        (overlay-put ov 'face 'jit-spell-misspelling)
+        (overlay-put ov 'category 'jit-spell)
       (jit-spell--unhide-overlay)
       (setq jit-spell--hidden-overlay
-            `(,(run-with-timer jit-spell-current-word-delay nil
-                               #'jit-spell--unhide-overlay)
-              ,ov
-              jit-spell-misspelling)))
+            (cons ov (run-with-timer jit-spell-current-word-delay nil
+                                     #'jit-spell--unhide-overlay))))
     ov))
 
 (defun jit-spell--make-overlays (buffer start end misspellings)
@@ -197,32 +179,31 @@ character offset from START, and a list of corrections."
 
 (defun jit-spell--search-overlay (pos count)
   "Return the COUNT jit-spell overlay from POS."
-  (let* ((limit (if (> count 0) (window-end) (window-start)))
-         (searchfn (if (> count 0)
+  (let* ((limit (if (cl-plusp count) (window-end) (window-start)))
+         (searchfn (if (cl-plusp count)
                        #'next-single-char-property-change
                      #'previous-single-char-property-change))
          (i (abs count)))
     (catch 'jit-spell
       (while (/= pos limit)
         (setq pos (funcall searchfn pos 'category nil limit))
-        (dolist (ov (overlays-at pos))
-          (when (eq (overlay-get ov 'category) 'jit-spell)
-            (cl-decf i)
-            (unless (< 0 i)
-              (throw 'jit-spell ov))))))))
+        (when-let ((ov (jit-spell--overlay-at pos)))
+          (unless (cl-plusp (cl-decf i))
+            (throw 'jit-spell ov)))))))
 
 (defun jit-spell--remove-overlays (start end &optional gaps)
   "Remove all `jit-spell' overlays between START and END, skipping GAPS.
 GAPS must be an ordered list of conses, with the intervals closer
 to END coming first."
-  (pcase-dolist (`(,i . ,j) gaps)
-    (dolist (ov (overlays-in j end))
-      (when (eq 'jit-spell (overlay-get ov 'category))
+  (named-let recur ((i (or (cdar gaps) start))
+                    (j end)
+                    (gaps gaps))
+    (dolist (ov (overlays-in i j))
+      (when (or (eq (overlay-get ov 'category) 'jit-spell)
+                (eq ov (car jit-spell--hidden-overlay)))
         (delete-overlay ov)))
-    (setq end i))
-  (dolist (ov (overlays-in start end))
-    (when (eq 'jit-spell (overlay-get ov 'category))
-      (delete-overlay ov))))
+    (when gaps
+      (recur (or (cdadr gaps) start) (caar gaps) (cdr gaps)))))
 
 (defun jit-spell--apply-correction (ov text)
   "Replace region spanned by OV with TEXT."
@@ -231,41 +212,36 @@ to END coming first."
     (delete-region (point) (overlay-end ov))
     (insert-before-markers text)))
 
-(defun jit-spell--context-menu (menu click)
-  "Context menu for `jit-spell-mode'.
-MENU and CLICK are as expected of a member of `context-menu-functions'.
-It can also be bound to a mouse click to pop up the menu."
-  (interactive "i\ne")
-  (save-excursion
-    (mouse-set-point click)
-    (when-let ((ov (jit-spell--overlay-at (point)))
-               (word (buffer-substring-no-properties
-                  (overlay-start ov) (overlay-end ov)))
-               (map (or menu (make-sparse-keymap))))
-      (dolist (corr (overlay-get ov 'jit-spell-corrections))
-        (easy-menu-add-item map '("Correct Word")
-                            (vector corr (lambda () (interactive)
-                                           (jit-spell--apply-correction ov corr)))))
-      (easy-menu-add-item map nil `["Save to Dictionary"
-                                    (jit-spell--accept-word ,word 'dict)])
-      (easy-menu-add-item map nil `["Save to Buffer"
-                                    (jit-spell--accept-word ,word 'buffer)])
-      (easy-menu-add-item map nil `["Accept for Session"
-                                    (jit-spell--accept-word ,word 'session)])
-      (unless menu (popup-menu map)))
-    menu))
-
 (defun jit-spell--unhide-overlay ()
   "Unhide the overlay stored in `jit-spell--hidden-overlay'."
-  (pcase jit-spell--hidden-overlay
-    (`(,timer ,ov ,face)
-     (cancel-timer timer)
-     (overlay-put ov 'face face)
-     (setq jit-spell--hidden-overlay nil))))
+  (when jit-spell--hidden-overlay
+    (overlay-put (car jit-spell--hidden-overlay) 'category 'jit-spell)
+    (cancel-timer (cdr jit-spell--hidden-overlay))
+    (setq jit-spell--hidden-overlay nil)))
+
+(defun jit-spell--unhide-overlay-maybe ()
+  "Unhide `jit-spell--hidden-overlay' if it doesn't overlap with word at point."
+  (when-let ((ov (car jit-spell--hidden-overlay)))
+    (unless (and (eq (overlay-buffer ov) (current-buffer))
+                 (save-excursion
+                   (re-search-forward (rx (or blank eol)))
+                   (<= (overlay-start ov) (match-beginning 0)))
+                 (save-excursion
+                   (re-search-backward (rx (or blank bol)))
+                   (<= (match-end 0) (overlay-end ov))))
+      (jit-spell--unhide-overlay))))
 
 (defun jit-spell--unfontify (&optional start end lax)
   "Remove overlays and forget checking status from START to END (or whole buffer).
 Force refontification of the region, unless LAX is non-nil."
+  ;; Drop pending requests for this buffer
+  (pcase-dolist (`(,_ . ,proc) jit-spell--process-pool)
+    (setf (process-get proc 'jit-spell--requests)
+          (seq-filter (lambda (r) (not (eq (car r) (current-buffer))))
+                      (process-get proc 'jit-spell--requests)))
+    (while (eq (car (process-get proc 'jit-spell--current-request))
+               (current-buffer))
+      (accept-process-output proc)))
   (without-restriction
     (setq start (or start (point-min)))
     (setq end (or end (point-max)))
@@ -297,7 +273,7 @@ The process plist includes the following properties:
 `jit-spell--requests': a FIFO queue with elements in the same
   form as above."
   (let* ((params (jit-spell--process-parameters))
-         (proc (plist-get jit-spell--process-pool params #'equal)))
+         (proc (cdr (assoc params jit-spell--process-pool))))
     (if (process-live-p proc)
         proc
       (unless ispell-async-processp
@@ -306,8 +282,7 @@ The process plist includes the following properties:
       (ispell-internal-change-dictionary)
       (setq proc (ispell-start-process))
       (set-process-query-on-exit-flag proc nil)
-      (setq jit-spell--process-pool
-            (plist-put jit-spell--process-pool params proc #'equal))
+      (setf (alist-get params jit-spell--process-pool nil nil #'equal) proc)
       (set-process-filter proc #'jit-spell--process-filter)
       (set-process-buffer proc (generate-new-buffer " *jit-spell*"))
       (process-send-string proc "!\n")  ;Enter terse mode
@@ -316,10 +291,8 @@ The process plist includes the following properties:
 (defun jit-spell--process-filter (proc string)
   "Filter function for jit-spell processes."
   (with-current-buffer (process-buffer proc)
-    (save-excursion
-      (goto-char (point-max))
-      (insert string))
-    (when (re-search-forward "^\n" nil t) ;TODO: Process in chunks
+    (insert string)
+    (when (and (eq (char-before) ?\n) (eq (pos-bol 0) (1- (point))))
       (pcase-let ((`(,buffer ,tick ,start ,end)
                    (process-get proc 'jit-spell--current-request)))
         (process-put proc 'jit-spell--current-request nil)
@@ -401,16 +374,26 @@ The process plist includes the following properties:
   "Return a list of subregions without control characters between START and END."
   (goto-char start)
   (let (regions)
-    (while (re-search-forward (rx (+ print)) end t)
+    (while (re-search-forward (rx (+ (not cntrl))) end t)
       (push (cons (match-beginning 0) (match-end 0)) regions))
     regions))
 
+;;; Mode-specific support
+
 (defun jit-spell--has-face-p (faces v)
   "Non-nil if V, a face or list of faces, includes any of the FACES."
   (if (listp v)
       (seq-some (lambda (f) (memq f faces)) v)
     (memq v faces)))
 
+(defun jit-spell--default-ignored-p (start end)
+  "Return non-nil if word between START and END should not be spell-checked."
+  (or (get-text-property start 'jit-spell-ignored)
+      (jit-spell--has-face-p jit-spell-ignored-faces
+                             (get-char-property start 'face))
+      (member (buffer-substring-no-properties start end)
+              jit-spell--local-words)))
+
 (defun jit-spell--refine-by-face (faces &optional only)
   "Return a function to refine a list of regions based on its faces.
 If ONLY is nil, regions containing any of the FACES are excluded.
@@ -420,32 +403,28 @@ Otherwise, only such regions are kept."
                 (lambda (faces v) (not (jit-spell--has-face-p faces v))))))
     (lambda (regions)
       (mapcan
-       (pcase-lambda (`(,i . ,limit)) ;; Refine one region
+       (pcase-lambda (`(,start . ,end)) ;; Refine one region
          (let (result)
-           (with-restriction i limit
-             (goto-char i)
+           (with-restriction start end
+             (goto-char start)
              (while-let ((prop (text-property-search-forward 'face faces pred)))
                (push `(,(prop-match-beginning prop) . ,(prop-match-end prop))
                      result)))
-           (jit-spell--remove-overlays i limit result)
+           (jit-spell--remove-overlays start end result)
            result))
        regions))))
 
 (defun jit-spell--apostrophe-hack (regions)
   "Refine REGIONS to work around Hunspell's apostrophe issue."
   (mapcan
-   (pcase-lambda (`(,i . ,limit)) ;; Refine one region
+   (pcase-lambda (`(,i . ,end)) ;; Refine one region
      (let (result)
        (goto-char i)
-       (while (re-search-forward
-               (rx (or (seq (or bol (not alpha)) (group ?') alpha)
-                       (seq alpha (group ?') (or (not alpha) eol))))
-               limit t)
-         (backward-char)
-         (let ((j (or (match-end 1) (match-beginning 2))))
-           (push (cons i j) result)
-           (setq i j)))
-       (push (cons i limit) result)))
+       (while (re-search-forward (rx (? (group alpha)) (group (+ ?'))) end t)
+         (unless (and (match-beginning 1) (looking-at-p (rx alpha)))
+           (push (cons i (match-beginning 2)) result)
+           (setq i (point))))
+       (push (cons i end) result)))
    regions))
 
 (defun jit-spell--check-region (start end)
@@ -453,10 +432,12 @@ Otherwise, only such regions are kept."
 This is intended to be a member of `jit-lock-functions'."
   (save-excursion
     ;; Extend region to include whole words
-    (goto-char start)
-    (setq start (if (re-search-backward "\\s-" nil t) (match-end 0) (point-min)))
-    (goto-char end)
-    (setq end (or (re-search-forward "\\s-" nil t) (point-max)))
+    (setq start (progn (goto-char start)
+                       (re-search-backward (rx (or blank bol)))
+                       (match-end 0)))
+    (setq end (progn (goto-char end)
+                     (re-search-forward (rx (or blank eol)))
+                     (match-beginning 0)))
     (let ((proc (jit-spell--get-process))
           (buffer (current-buffer))
           (tick (buffer-chars-modified-tick)))
@@ -468,7 +449,7 @@ This is intended to be a member of `jit-lock-functions'."
             (jit-spell--send-request proc request)))))
     `(jit-lock-bounds ,start . ,end)))
 
-;;; Interactive commands and major mode
+;;; Interactive commands
 
 (defun jit-spell--accept-word (word where)
   "Accept spelling of WORD.
@@ -487,63 +468,95 @@ the above)."
                                 (format "*%s\n#\n" word)))
     ('query (jit-spell--accept-word
              word
-             (pcase (read-multiple-choice (substitute-quotes
-                                           (format "Add `%s' to" word))
-                                          '((?d "dictionary")
-                                            (?b "buffer")
-                                            (?s "session"))) ;TODO: help string
-               (`(?d ,_) 'dict)
-               (`(?b ,_) 'buffer)
-               (`(?s ,_) 'session)))))
+             (pcase (read-multiple-choice
+                     (substitute-quotes (format "Add `%s' to" word))
+                     '((?d "dictionary" "Save this word in your personal dictionary.")
+                       (?b "buffer" "Add a local word at the end of this buffer.")
+                       (?s "session" "Accept this spelling temporarily and in this buffer only.")))
+               (`(?d . ,_) 'dict)
+               (`(?b . ,_) 'buffer)
+               (`(?s . ,_) 'session)))))
   (jit-lock-refontify))
-(define-obsolete-function-alias 'jit-spell-accept-word 'jit-spell--accept-word "0.2")
 
-(defun jit-spell-correct-word--next (arg)
-  "Perform a spooky action at a distance."
-  (interactive "p")
-  (throw 'jit-spell-correct-word--next arg))
-
-(defun jit-spell-correct-word (arg &optional pos)
+(defun jit-spell-correct-word (arg)
   "Correct a misspelled word in the selected window.
 
 You can also accept the spelling in question by entering `@' in
-the prompt.  It is possible to modify the spelling to be
-accepted, say change capitalization or inflection, by entering
-any text after the `@'.
+the prompt.  To modify the spelling to be accepted, say change
+capitalization or inflection, enter any text after the `@'.
 
-With a numeric ARG, move backwards that many misspellings.
-Alternatively, pressing \\<jit-spell-mode-map>\\[jit-spell-correct-word] \
-again moves to the next misspelling."
+In the minibuffer, cycle between the visible misspellings.  There
+command is bound to `jit-spell-correct-next-key'.
+
+With a numeric ARG, move backwards that many misspellings."
   (interactive "p")
-  (let* ((ov (or (jit-spell--search-overlay (or pos (point)) (- arg))
-                 (user-error "No more misspellings")))
-         (start (overlay-start ov))
-         (end (overlay-end ov))
-         (word (buffer-substring-no-properties start end))
-         (highlight (make-overlay start end))
-         (map (make-sparse-keymap))
-         (prompt (format-prompt "Correct `%s' (`@' to accept)" nil word))
-         (coll (append (overlay-get ov 'jit-spell-corrections)
-                       (list (concat "@" word)))))
-    (dolist (key (where-is-internal 'jit-spell-correct-word))
-      (define-key map key 'jit-spell-correct-word--next))
-    (overlay-put highlight 'face 'highlight)
-    (pcase (catch 'jit-spell-correct-word--next
-             (minibuffer-with-setup-hook
-                 (lambda ()
-                   (set-keymap-parent map (current-local-map))
-                   (use-local-map map))
-               (unwind-protect
-                   (completing-read prompt coll nil nil nil nil nil t)
-                 (delete-overlay highlight))))
-      ((and count (pred numberp))
-       (jit-spell-correct-word count (overlay-start ov)))
-      ((and corr (rx bos ?@ (* space) (? (group (+ nonl)))))
-       (jit-spell--accept-word (or (match-string 1 corr) word) 'query))
-      (corr (jit-spell--apply-correction ov corr)))))
+  (if (minibufferp)
+      (throw 'jit-spell--next arg)
+    (named-let recur ((count (- arg)) (pos (point)))
+      (let* ((ov (or (jit-spell--search-overlay pos count)
+                     (user-error "No more misspellings")))
+             (start (overlay-start ov))
+             (end (overlay-end ov))
+             (word (buffer-substring-no-properties start end))
+             (highlight (make-overlay start end))
+             (key (if (eq jit-spell-correct-next-key 'auto)
+                      (where-is-internal 'jit-spell-correct-word nil t)
+                    jit-spell-correct-next-key))
+             (prompt (format-prompt "Correct `%s' (`@' to accept)" nil word))
+             (cands (append (overlay-get ov 'jit-spell-corrections)
+                            (list (concat "@" word))))
+             (metadata '(metadata (category . jit-spell)
+                                  (cycle-sort-function . identity)
+                                  (display-sort-function . identity)))
+             (coll (lambda (string pred action)
+                     (if (eq action 'metadata)
+                         metadata
+                       (complete-with-action action cands string pred)))))
+        (overlay-put highlight 'face 'highlight)
+        (pcase (catch 'jit-spell--next
+                 (minibuffer-with-setup-hook
+                     (lambda ()
+                       (when-let ((map (and key (make-sparse-keymap))))
+                         (set-keymap-parent map (current-local-map))
+                         (define-key map key 'jit-spell-correct-word)
+                         (use-local-map map)))
+                   (unwind-protect
+                       (completing-read prompt coll nil nil nil nil nil t)
+                     (delete-overlay highlight))))
+          ((and arg (pred numberp))
+           (recur (* (cl-signum count) arg) (overlay-start ov)))
+          ((and corr (rx bos ?@ (* ?\s) (? (group (+ nonl)))))
+           (jit-spell--accept-word (or (match-string 1 corr) word) 'query))
+          (corr (jit-spell--apply-correction ov corr)))))))
 
 (defalias 'jit-spell-change-dictionary 'ispell-change-dictionary) ;For discoverability
 
+;;; Minor mode definition
+
+(defun jit-spell--context-menu (menu click)
+  "Context menu for `jit-spell-mode'.
+MENU and CLICK are as expected of a member of `context-menu-functions'.
+It can also be bound to a mouse click to pop up the menu."
+  (interactive "i\ne")
+  (save-excursion
+    (mouse-set-point click)
+    (when-let ((ov (jit-spell--overlay-at (point)))
+               (word (buffer-substring-no-properties
+                  (overlay-start ov) (overlay-end ov)))
+               (map (or menu (make-sparse-keymap))))
+      (dolist (corr (overlay-get ov 'jit-spell-corrections))
+        (easy-menu-add-item map '("Correct Word")
+                            (vector corr (lambda () (interactive)
+                                           (jit-spell--apply-correction ov corr)))))
+      (easy-menu-add-item map nil `["Save to Dictionary"
+                                    (jit-spell--accept-word ,word 'dict)])
+      (easy-menu-add-item map nil `["Save to Buffer"
+                                    (jit-spell--accept-word ,word 'buffer)])
+      (easy-menu-add-item map nil `["Accept for Session"
+                                    (jit-spell--accept-word ,word 'session)])
+      (unless menu (popup-menu map)))
+    menu))
+
 (defun jit-spell--read-local-words ()
   "Look for local words in the buffer and accept them for this session."
   (save-excursion
@@ -553,22 +566,15 @@ again moves to the next misspelling."
 	(while (re-search-forward "\\s-*\\(\\S-+\\)" limit t)
           (jit-spell--accept-word (match-string-no-properties 1) 'session))))))
 
-(defvar-keymap jit-spell-mode-map :doc "Keymap for `jit-spell-mode'.")
-
 ;;;###autoload
 (define-minor-mode jit-spell-mode
   "Just-in-time spell checking."
+  :keymap (make-sparse-keymap)
   :lighter (" Spell"
             (:propertize
              (:eval
-	      (concat "/" (let ((s (or ispell-local-dictionary
-			               ispell-dictionary
-                                       "--")))
-                            (substring s 0 (string-search "_" s)))))
-             help-echo "mouse-1: Change dictionary"
-             local-map (keymap
-                        (mode-line keymap
-                                   (mouse-1 . ispell-change-dictionary)))))
+              (let ((s (or ispell-local-dictionary ispell-dictionary)))
+                (if s (concat "/" (substring s 0 (string-search "_" s))))))))
   (cond
    (jit-spell-mode
     ;; Major mode support
@@ -576,7 +582,7 @@ again moves to the next misspelling."
      ((derived-mode-p 'prog-mode)
       (add-function :filter-return (local 'jit-spell--filter-region)
                     (jit-spell--refine-by-face jit-spell-prog-mode-faces t)))
-     ((derived-mode-p 'tex-mode)
+     ((derived-mode-p 'tex-mode 'context-mode)
       (add-function :filter-return (local 'jit-spell--filter-region)
                     (jit-spell--refine-by-face jit-spell-tex-ignored-faces))))
     ;; Generic ignore predicate
@@ -597,11 +603,13 @@ again moves to the next misspelling."
     (jit-spell--read-local-words)
     (add-hook 'ispell-change-dictionary-hook 'jit-spell--unfontify nil t)
     (add-hook 'context-menu-functions 'jit-spell--context-menu nil t)
+    (add-hook 'post-command-hook #'jit-spell--unhide-overlay-maybe nil t)
     ;; Font-lock needs to run first
     (add-hook 'jit-lock-functions #'jit-spell--check-region 10 t)
     (jit-lock-register #'jit-spell--check-region))
    (t
     (jit-lock-unregister #'jit-spell--check-region)
+    (remove-hook 'post-command-hook #'jit-spell--unhide-overlay-maybe t)
     (remove-hook 'context-menu-functions 'jit-spell--context-menu t)
     (remove-hook 'ispell-change-dictionary-hook 'jit-spell--unfontify t)
     (kill-local-variable 'jit-spell--local-words)
@@ -610,15 +618,12 @@ again moves to the next misspelling."
     (jit-spell--unfontify nil nil t))))
 
 ;; Don't litter M-x
-(dolist (sym '(jit-spell--context-menu
-               jit-spell-correct-word--next))
-  (put sym 'completion-predicate #'ignore))
-
+(put 'jit-spell--context-menu 'completion-predicate #'ignore)
 (dolist (sym '(jit-spell-change-dictionary
                jit-spell-correct-word))
-  (put sym 'completion-predicate (lambda (_ buffer)
-                                   (buffer-local-value 'jit-spell-mode
-                                                       buffer))))
+  (put sym 'completion-predicate
+       (lambda (_ buffer)
+         (buffer-local-value 'jit-spell-mode buffer))))
 
 (provide 'jit-spell)
 
